"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.envEnd = void 0;
/* Hand-written tokenizers for TeX. */
const lezer_1 = require("lezer");
const escape_regex_1 = __importDefault(require("./utils/escape-regex"));
const terms_1 = require("./terms");
const leftBrace = "{".charCodeAt(0);
const rightBrace = "}".charCodeAt(0);
const end = Uint8Array.from("\\end", (c) => c.charCodeAt(0));
exports.envEnd = new lezer_1.ExternalTokenizer((input, token, stack) => {
    let pos = token.start;
    let tokEnd = pos;
    for (let next = input.get(pos), i = 0; next; i += 1, next = input.get(pos)) {
        if (next !== end[i] && i < end.length)
            return;
        if (i === end.length)
            break;
        else {
            pos += 1;
            tokEnd = pos;
        }
    }
    pos += 1;
    if (input.get(pos) !== leftBrace)
        return;
    pos += 1;
    const nameStart = pos;
    let nestLevel = 1;
    for (let next = input.get(pos); next >= 0; pos += 1, next = input.get(pos)) {
        if (next === leftBrace)
            nestLevel += 1;
        if (next === rightBrace)
            nestLevel -= 1;
        if (nestLevel === 0)
            break;
        pos += 1;
    }
    if (pos > nameStart + 1) {
        const name = input.read(nameStart, pos);
        const contextStart = stack.startOf([terms_1.Environment]);
        if (contextStart >= 0) {
            if (!new RegExp(escape_regex_1.default(name)).exec(input.read(contextStart, contextStart + name.length + 14))) {
                token.accept(terms_1.MismatchedEnvEndCommand, tokEnd);
                return;
            }
        }
    }
    token.accept(terms_1.EnvEndCommand, tokEnd);
}, { contextual: true });
